"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initValidator = exports.DEFAULT_VALIDATOR_CONFIG = void 0;
const utils_1 = require("../utils");
const http_1 = require("../utils/http");
const child_process_1 = require("child_process");
const prepare_config_1 = require("./prepare-config");
const ensure_validator_up_1 = require("./ensure-validator-up");
const server_1 = require("../relay/server");
const types_1 = require("../relay/types");
const storage_1 = require("../storage");
const local_accounts_1 = require("../assets/local-accounts");
const types_2 = require("../assets/types");
const path_1 = __importDefault(require("path"));
const fs_1 = require("../utils/fs");
/**
 * @private
 */
exports.DEFAULT_VALIDATOR_CONFIG = {
    killRunningValidators: true,
    programs: [],
    accountsCluster: 'https://metaplex.devnet.rpcpool.com',
    accounts: [],
    jsonRpcUrl: utils_1.LOCALHOST,
    websocketUrl: '',
    commitment: 'singleGossip',
    ledgerDir: (0, utils_1.tmpLedgerDir)(),
    resetLedger: true,
    limitLedgerSize: 1e4,
    verifyFees: false,
    detached: process.env.CI != null,
};
/**
 * @private
 */
async function initValidator(validatorConfig, relayConfig = {}, storageConfig, assetsFolder = types_2.DEFAULT_ASSETS_FOLDER, forceClone) {
    const { killRunningValidators, programs, accountsCluster, accounts, jsonRpcUrl, websocketUrl, commitment, ledgerDir, resetLedger, limitLedgerSize, verifyFees, detached, } = { ...exports.DEFAULT_VALIDATOR_CONFIG, ...validatorConfig };
    const { killRunningRelay, accountProviders, accountRenderers, enabled: relayEnabled, } = {
        ...types_1.DEFAULT_RELAY_CONFIG,
        ...relayConfig,
    };
    if (killRunningValidators) {
        try {
            (0, child_process_1.execSync)('pkill -f solana-test-validator');
            (0, utils_1.logInfo)('Killed currently running solana-test-validator');
            await (0, utils_1.sleep)(1000);
        }
        catch (err) { }
    }
    const { configPath, cleanupConfig } = await (0, prepare_config_1.solanaConfig)({
        websocketUrl,
        jsonRpcUrl,
        commitment,
    });
    const args = ['--quiet', '-C', configPath, '--ledger', ledgerDir];
    if (resetLedger)
        args.push('-r');
    if (programs.length > 0) {
        for (const { programId, deployPath } of programs) {
            if (!(0, fs_1.canAccessSync)(deployPath)) {
                throw new Error(`Cannot access program deploy path of ${deployPath}`);
            }
            args.push('--bpf-program');
            args.push(programId);
            args.push(deployPath);
        }
    }
    if (accounts.length > 0) {
        const accountsFolder = path_1.default.resolve(process.cwd(), path_1.default.join(assetsFolder, types_2.ACCOUNTS_FOLDER));
        await (0, local_accounts_1.handleFetchAccounts)(accountsCluster, accounts, accountsFolder, forceClone);
        for (const { accountId, executable, cluster } of accounts) {
            const accountPath = path_1.default.join(accountsFolder, `${accountId}.json`);
            if (await (0, fs_1.canAccess)(accountPath)) {
                args.push('--account');
                args.push(accountId);
                args.push(accountPath);
            }
            else {
                throw new Error(`Can't find account info file for account ${accountId} cloned from cluster ${cluster !== null && cluster !== void 0 ? cluster : accountsCluster}! \nMake sure the account exists on that cluster and try again.`);
            }
            if (executable) {
                const executableId = await (0, local_accounts_1.getExecutableAddress)(accountId);
                const executablePath = path_1.default.join(accountsFolder, `${executableId}.json`);
                if (await (0, fs_1.canAccess)(executablePath)) {
                    args.push('--account');
                    args.push(executableId);
                    args.push(executablePath);
                }
                else {
                    (0, utils_1.logError)(`Can't find executable account info file for executable account ${accountId}`);
                }
            }
        }
    }
    args.push(...['--limit-ledger-size', limitLedgerSize.toString()]);
    const cmd = `solana-test-validator ${args.join(' \\\n  ')}`;
    if (utils_1.logTrace.enabled) {
        (0, utils_1.logTrace)('Launching validator with the following command');
        console.log(cmd);
    }
    const child = (0, child_process_1.spawn)('solana-test-validator', args, {
        detached,
        stdio: 'inherit',
    });
    child.unref();
    await new Promise((resolve, reject) => {
        child.on('spawn', resolve).on('error', reject);
    });
    (0, utils_1.logInfo)('Spawning new solana-test-validator with programs predeployed and ledger at %s', ledgerDir);
    // Launch relay server in parallel
    if (relayEnabled) {
        server_1.Relay.startServer(accountProviders, accountRenderers, programs, accounts, killRunningRelay)
            .then(({ app }) => {
            (0, utils_1.logInfo)('Successfully launched Relay at %s', (0, http_1.resolveServerAddress)(app));
        })
            .catch((err) => {
            const msg = 'Failed to launch Relay';
            (0, utils_1.logError)(msg);
            (0, utils_1.logError)(err);
        });
    }
    // Launch Storage server in parallel as well
    const storageEnabled = storageConfig != null &&
        { ...storage_1.DEFAULT_STORAGE_CONFIG, ...storageConfig }.enabled;
    if (storageEnabled) {
        (0, http_1.killRunningServer)(storage_1.AMMAN_STORAGE_PORT)
            .then(() => storage_1.MockStorageServer.createInstance(storageConfig).then((storage) => storage.start()))
            .then((server) => {
            (0, utils_1.logInfo)('Successfully launched MockStorageServer at %s', (0, http_1.resolveServerAddress)(server));
        })
            .catch((err) => {
            const msg = 'Failed to launch MockStorageServer';
            (0, utils_1.logError)(msg);
            (0, utils_1.logError)(err);
        });
    }
    await (0, ensure_validator_up_1.ensureValidatorIsUp)(jsonRpcUrl, verifyFees);
    await cleanupConfig();
    (0, utils_1.logInfo)('solana-test-validator is up');
}
exports.initValidator = initValidator;
//# sourceMappingURL=init-validator.js.map